@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2003-2006 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2003-2007 Hewlett-Packard Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,13 +20,13 @@
 #
 
 # Std Lib
-import sys, struct
+import sys
+import struct
 
 # Local
 from g import *
 from base.utils import unprintable
 
-
 # Request codes
 GET_REQUEST = 0x00
 GET_NEXT_REQUEST = 0x01
@@ -289,6 +289,7 @@ def parsePMLPacket(p, expected_data_type=TYPE_UNKNOWN):
     return data, data_type, error_code
 
 
+
 def HPToSNMP(oid): # 1.
     return '.'.join(['1.3.6.1.4.1.11.2.3.9.4.2', oid, '0'])
 
@@ -319,6 +320,9 @@ def PMLToSNMP(oid):
 #
 # OIDs
 #
+
+OID_DEVICE_ID = ('0.1.3.6.1.4.1.11.2.3.9.1.1.7.0', TYPE_STRING) # network/SNMP only (undocumented)
+
 OID_DEVICE_SUPPORTED_FUNCTIONS = ('1.1.2.67', TYPE_COLLECTION)
 DEVICE_SUPPORTED_FUNCTIONS_SCAN =                 0x00002
 DEVICE_SUPPORTED_FUNCTIONS_SCAN_SIMPLEX =         0x00004
@@ -353,8 +357,11 @@ PRINT_INTERNAL_PAGE_SUPPLIES_PAGE = 101
 PRINT_INTERNAL_PAGE_COLOR_PALETTE_CMYK_PAGE = 259
 PRINT_INTERNAL_PAGE_COLOR_CAL = 1102
 PRINT_INTERNAL_PAGE_COLOR_CAL_VERIFICATION = 1104
-PRINT_INTERNAL_PAGE_ALIGNMENT_PAGE = 1100 # LBOW
+PRINT_INTERNAL_PAGE_ALIGNMENT_PAGE = 1100 # LBOW/OJ Pro L7xxx
 PRINT_INTERNAL_PAGE_ALIGNMENT_PAGE_VERIFICATION = 1150 # LBOW
+PRINT_INTERNAL_PAGE_LINEFEED_CALIBRATION = 1407 # OJ Pro
+PRINT_INTERNAL_PAGE_AUTOMATIC_COLOR_CALIBRATION = 1408 # PS Pro B8800
+PRINT_INTERNAL_PAGE_PRINT_QUALITY_DIAGNOSTIC = 1409 # OJ Pro
 
 # From xojpanel
 OID_SPM_LINE1 = ('2.16.5.1.2.1.1', TYPE_STRING)
@@ -505,6 +512,17 @@ POWER_MODE_CHARGING = 0x04
 POWER_MODE_DISCHARGING = 0x08
 POWER_MODE_BATTERY_LEVEL_KNOWN = 0x10
 
+OID_BATTERY_LEVEL_2 = ('1.1.2.61', TYPE_SIGNED_INTEGER)
+
+OID_POWER_SETTINGS = ('1.1.2.118', TYPE_ENUMERATION)
+OID_POWER_SETTINGS_15MIN = 1
+OID_POWER_SETTINGS_30MIN = 2
+OID_POWER_SETTINGS_45MIN = 3
+OID_POWER_SETTINGS_1HR = 4
+OID_POWER_SETTINGS_2HR = 5
+OID_POWER_SETTINGS_3HR = 6
+OID_POWER_SETTINGS_NEVER = 999
+
 #
 # Fax
 #
@@ -560,6 +578,44 @@ UPDN_STATE_STR = {UPDN_STATE_IDLE: "Idle",
                   UPDN_STATE_NEWPAGE: "New page",
                   UPDN_STATE_DISABLED: "Disabled",}
 
+# Fax download errors (taken from doc/Fax/WindowsFax/sdcore/DevIO/DevIODefs.h)
+#
+DN_ERROR_NONE = 0
+DN_ERROR_HOST_ABORT = 705
+DN_ERROR_STOP_KEY_PRESSED = 706
+DN_ERROR_SESSION_FAIL = 709
+DN_ERROR_TX_ERROR = 710
+DN_ERROR_PHONE_UNAVAILABLE = 711
+DN_ERROR_OUT_OF_MEMORY = 713
+DN_ERROR_RESULT_BUSY = 714
+DN_ERROR_NO_ANSWER = 715
+DN_ERROR_NO_DIAL_TONE = 716
+DN_ERROR_DOC_JAM = 717
+DN_ERROR_DOOR_OPEN = 718
+DN_ERROR_POWER_FAILED = 719
+DN_ERROR_BLACKLIST = 720
+DN_ERROR_DOC_STORAGE_FULL = 721
+DN_ERROR_RESULT_COLOR_UNSUP = 722
+DN_ERROR_UNKNOWN = 9999
+
+DN_ERROR_STR = {DN_ERROR_NONE: "None",
+                DN_ERROR_HOST_ABORT: "Host aborted",
+                DN_ERROR_STOP_KEY_PRESSED: "Stop key pressed",
+                DN_ERROR_SESSION_FAIL: "Session failed",
+                DN_ERROR_TX_ERROR: "Transmit error",
+                DN_ERROR_PHONE_UNAVAILABLE: "Phone unavailable",
+                DN_ERROR_OUT_OF_MEMORY: "Out of memory",
+                DN_ERROR_RESULT_BUSY: "Result busy",
+                DN_ERROR_NO_ANSWER: "No answer",
+                DN_ERROR_NO_DIAL_TONE: "No dial tone",
+                DN_ERROR_DOC_JAM: "Document jammed",
+                DN_ERROR_DOOR_OPEN: "Door open",
+                DN_ERROR_POWER_FAILED: "Power failed",
+                DN_ERROR_BLACKLIST: "Blacklisted",
+                DN_ERROR_DOC_STORAGE_FULL: "Document storage full",
+                DN_ERROR_RESULT_COLOR_UNSUP: "Color unsupported",
+                DN_ERROR_UNKNOWN: "Unknown error"}
+
 OID_FAX_TOKEN = ('1.1.1.27', TYPE_BINARY)
 
 OID_FAX_TX_ID = ('1.1.6.3.2.3', TYPE_SIGNED_INTEGER)
@@ -656,9 +712,68 @@ COPIER_JOB_ACTIVE = 3
 COPIER_JOB_ABORTING = 4
 COPIER_JOB_SETUP = 5
 
+# AiO Specific Copy
+
+OID_SCAN_TO_PRINTER = ('1.5.1.5', TYPE_SIGNED_INTEGER)
+SCAN_TO_PRINTER_IDLE         = 1
+SCAN_TO_PRINTER_START        = 2
+SCAN_TO_PRINTER_ACTIVE       = 3
+SCAN_TO_PRINTER_ABORTED      = 4
+SCAN_TO_PRINTER_SET_DEFAULTS = 5
+SCAN_TO_PRINTER_GET_DEFAULTS = 6
+
+OID_PIXEL_DATA_TYPE = ('1.5.1.3', TYPE_SIGNED_INTEGER)
+PIXEL_DATA_TYPE_GRAYSCALE_256     = 8
+PIXEL_DATA_TYPE_COLOR_24_BIT      = 24
+
+OID_COPIER_SPECIAL_FEATURES = ('1.5.1.16', TYPE_SIGNED_INTEGER)
+COPY_FEATURE_NONE     = 1
+COPY_FEATURE_CLONE    = 2
+COPY_FEATURE_POSTER   = 3
+COPY_FEATURE_MIRROR   = 4
+COPY_FEATURE_AUTOFIT  = 5
+COPY_FEATURE_TWOUP    = 6
+COPY_FEATURE_AUTOFILL = 7
+
+
+OID_COPIER_PHOTO_MODE = ('1.5.1.15', TYPE_COLLECTION)
+ENHANCE_LIGHT_COLORS  =  0x00000001  # Bit  0
+ENHANCE_TEXT          =  0x00000002  # Bit  1
+
+OID_COPIER_NUM_COPIES_AIO = ('1.5.1.6', TYPE_SIGNED_INTEGER)
+OID_COPIER_CONTRAST_AIO = ('1.5.1.2', TYPE_SIGNED_INTEGER)
+OID_COPIER_REDUCTION_AIO = ('1.5.1.4', TYPE_SIGNED_INTEGER)
+OID_COPIER_QUALITY_AIO = ('1.5.1.13', TYPE_ENUMERATION)
+
+OID_COPIER_JOB_INPUT_TRAY_SELECT = ('1.5.1.51', TYPE_SIGNED_INTEGER)
+COPIER_JOB_INPUT_TRAY_1 = 1
+COPIER_JOB_INPUT_TRAY_2 = 1
+COPIER_JOB_INPUT_TRAY_3 = 1
+
+OID_COPIER_MEDIA_TYPE = ('1.5.1.14', TYPE_SIGNED_INTEGER)
+COPIER_MEDIA_TYPE_PLAIN               = 1
+COPIER_MEDIA_TYPE_BRIGHT_WHITE        = 2
+COPIER_MEDIA_TYPE_PREMIUM_PHOTO       = 3
+COPIER_MEDIA_TYPE_SPECIAL             = 4
+COPIER_MEDIA_TYPE_TRANSPARENCY        = 5
+COPIER_MEDIA_TYPE_IRON_ON             = 6
+COPIER_MEDIA_TYPE_FAST_TRANSPARANCEY  = 7
+COPIER_MEDIA_TYPE_BROCHURE_MATTE      = 8
+COPIER_MEDIA_TYPE_BROCHURE_GLOSSY     = 9
+COPIER_MEDIA_TYPE_PHOTO_GLOSSY        = 10
+COPIER_MEDIA_TYPE_MATTE_PAPER         = 11
+COPIER_MEDIA_TYPE_EVERYDAY_PHOTO      = 12
+COPIER_MEDIA_TYPE_PHOTO_QUAL_INKJET   = 13
+COPIER_MEDIA_TYPE_PHOTO               = 14 
+COPIER_MEDIA_TYPE_AUTOMATIC           = 15
+COPIER_MEDIA_TYPE_ADVANCED_PHOTO      = 16
+COPIER_MEDIA_TYPE_IRON_ON_MIRRORED    = 17
 
 
 
+# Misc
+
+OID_DATE_AND_TIME = ('1.1.2.17', TYPE_BINARY)
 
 
 
