@@ -634,6 +634,52 @@ bugout:
    return stat;
 }
 
+/* See if this usb device and serial number match. Return model if match. */
+static int is_serial(struct usb_device *dev, const char *sn, char *model, int model_size)
+{
+   usb_dev_handle *hd=NULL;
+   char sz[128];
+   char gen[128];
+   int r, stat=0;
+
+   if ((hd = usb_open(dev)) == NULL)
+   {
+      BUG("invalid usb_open: %m\n");
+      goto bugout;
+   }
+
+   if (dev->descriptor.idVendor != 0x3f0)
+      goto bugout;      /* not a HP product */
+
+   if ((r=get_string_descriptor(hd, dev->descriptor.iSerialNumber, sz, sizeof(sz))) < 0)
+   {
+      BUG("invalid serial id string ret=%d\n", r);
+      goto bugout;
+   }
+   if (sz[0])
+      generalize_serial(sz, gen, sizeof(gen));
+   else
+      strcpy(gen, "0");
+
+   if (strncmp(sn, gen, sizeof(gen)) != 0)
+      goto bugout;  /* match failed */
+
+   if ((r=get_string_descriptor(hd, dev->descriptor.iProduct, sz, sizeof(sz))) < 0)
+   {
+      BUG("invalid product id string ret=%d\n", r);
+      goto bugout;
+   }
+   generalize_model(sz, model, model_size);
+
+   stat = 1;    /* found usb device that matches sn */
+     
+bugout:
+   if (hd != NULL)
+      usb_close(hd);
+
+   return stat;
+}
+
 static struct usb_device *get_libusb_device(const char *uri)
 {
    struct usb_bus *bus;
@@ -843,7 +889,7 @@ static int new_channel(mud_device *pd, int index, const char *sn)
       goto bugout; 
    }
 
-   if (index == HPMUD_EWS_CHANNEL || index == HPMUD_SOAPSCAN_CHANNEL || index == HPMUD_SOAPFAX_CHANNEL)
+   if (index == HPMUD_EWS_CHANNEL || index == HPMUD_SOAPSCAN_CHANNEL || index == HPMUD_SOAPFAX_CHANNEL || index == HPMUD_MARVELL_SCAN_CHANNEL)
       pd->channel[index].vf = musb_comp_channel_vf;
    else if (pd->io_mode == HPMUD_RAW_MODE || pd->io_mode == HPMUD_UNI_MODE)
       pd->channel[index].vf = musb_raw_channel_vf;
@@ -1261,10 +1307,9 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) musb_channel_open(mud_
    else
    {
       if ((stat = (pd->channel[index].vf.open)(&pd->channel[index])) != HPMUD_R_OK)  /* call transport specific open */
-      {
          del_channel(pd, &pd->channel[index]);   /* open failed, cleanup */
-      }
-      *cd = index;
+      else
+         *cd = index;
    }
 
    pthread_mutex_unlock(&pd->mutex);
@@ -1408,6 +1453,9 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) musb_comp_channel_open
       case HPMUD_SOAPFAX_CHANNEL:
          fd = FD_ff_3_1;   
          break;
+      case HPMUD_MARVELL_SCAN_CHANNEL:
+         fd = FD_ff_ff_ff;   
+         break;
       default:
          stat = HPMUD_R_INVALID_SN;
          BUG("invalid %s channel=%d\n", pc->sn, pc->index);
@@ -2064,4 +2112,37 @@ bugout:
    return stat;
 }
 
+enum HPMUD_RESULT hpmud_make_usb_serial_uri(const char *sn, char *uri, int uri_size, int *bytes_read)
+{
+   struct usb_bus *bus;
+   struct usb_device *dev, *found_dev=NULL;
+   char model[128];
+   enum HPMUD_RESULT stat = HPMUD_R_INVALID_DEVICE_NODE;
+
+   DBG("[%d] hpmud_make_usb_serial_uri() sn=%s\n", getpid(), sn);
+
+   *bytes_read=0;
+
+   usb_init();
+   usb_find_busses();
+   usb_find_devices();
+
+   for (bus=usb_busses; bus && !found_dev; bus=bus->next)
+      for (dev=bus->devices; dev && !found_dev; dev=dev->next)
+	if (is_serial(dev, sn, model, sizeof(model)))
+            found_dev = dev;  /* found usb device that matches serial number */
+
+   if (found_dev == NULL)
+   {
+      BUG("invalid sn %s\n", sn);
+      goto bugout;
+   }
+ 
+   *bytes_read = snprintf(uri, uri_size, "hp:/usb/%s?serial=%s", model, sn); 
+   stat = HPMUD_R_OK;
+
+bugout:
+   return stat;
+}
+
 
