@@ -217,7 +217,7 @@ PAPER_SIZE DJGenericVIP::MandatoryPaperSize ()
  *      100 - 13 X 19 size
  */
 
-    if (pSS->IOMode.bDevID && (pSS->GetDeviceID (sDevIdStr, DevIDBuffSize, FALSE)) == NO_ERROR)
+    if ((pSS->GetDeviceID (sDevIdStr, DevIDBuffSize, FALSE)) == NO_ERROR)
     {
         if ((pStr = strstr ((char *) sDevIdStr, ";S:")) && (pSS->GetVIPVersion ()) >= 3)
         {
@@ -255,7 +255,7 @@ BOOL DJGenericVIP::FullBleedCapable (PAPER_SIZE ps, FullbleedType  *fbType, floa
     char    *pStr;
     sDevIdStr[0] = 0;
 
-    if (pSS->IOMode.bDevID && (pSS->GetDeviceID (sDevIdStr, DevIDBuffSize, FALSE)) == NO_ERROR)
+    if ((pSS->GetDeviceID (sDevIdStr, DevIDBuffSize, FALSE)) == NO_ERROR)
     {
         if ((pStr = strstr ((char *) sDevIdStr, ";S:")) && (pSS->GetVIPVersion ()) >= 3)
         {
@@ -438,7 +438,7 @@ BOOL DJGenericVIP::HagakiFeedPresent(BOOL bQueryPrinter)
     // skip over ";S:<version=2bytes><topcover><inklid><duplexer>"
     pStr += 8;
     BYTE b = *pStr;
-    return (b & 4 == 4);
+    return ((b & 4) == 4);
 }
 
 #ifdef APDK_AUTODUPLEX
@@ -455,7 +455,7 @@ BOOL DJGenericVIP::HagakiFeedDuplexerPresent(BOOL bQueryPrinter)
         bQueryPrinter = FALSE;
     }
 
-    err = pSS->GetDeviceID(bDevIDBuff, DevIDBuffSize, bQueryPrinter);
+    err = pSS->GetDeviceID (bDevIDBuff, DevIDBuffSize, bQueryPrinter);
     if (err!=NO_ERROR)
     {
         return 0;
@@ -469,7 +469,7 @@ BOOL DJGenericVIP::HagakiFeedDuplexerPresent(BOOL bQueryPrinter)
     // skip over ";S:<version=2bytes><topcover><inklid>"
     pStr += 6;
     BYTE b = *pStr;
-    return (b & 4 == 4);
+    return ((b & 4) == 4);
 }
 #endif
 
