@@ -123,6 +123,8 @@ public:
         "Deskjet 6988\0"
 		"deskjet 9600\0"
 		"Deskjet 9800\0"
+        "Deskjet D25\0"
+        "Deskjet F42\0"
         "Business Inkjet 1000\0"
 		"hp business inkjet 1100\0"
    		"HP Business Inkjet 1200\0"
@@ -164,6 +166,7 @@ public:
         "Officejet 63\0"
         "Officejet J35\0"
         "Officejet J36\0"
+        "Officejet J46\0"
         "Officejet J55\0"
         "Officejet J57\0"
         "Officejet J64\0"
@@ -178,22 +181,28 @@ public:
         "Photosmart A310\0"
         "Photosmart A320\0"
         "Photosmart A440\0"
+        "Photosmart B85\0"
         "Photosmart Pro B8300\0"
         "Photosmart Pro B88\0"
         "Photosmart C31\0"
         "Photosmart C41\0"
         "Photosmart C42\0"
         "Photosmart C43\0"
+        "Photosmart C44\0"
+        "Photosmart C45\0"
         "Photosmart C51\0"
         "Photosmart C52\0"
+        "Photosmart C53\0"
         "Photosmart C61\0"
         "Photosmart C62\0"
+        "Photosmart C63\0"
         "Photosmart C71\0"
         "Photosmart C72\0"
         "Photosmart C81\0"
         "Photosmart D5060\0"
         "Photosmart D51\0"
         "Photosmart D53\0"
+        "Photosmart D54\0"
         "Photosmart D61\0"
         "Photosmart D71\0"
         "Photosmart D72\0"
