@@ -386,6 +386,7 @@ class LJZjsMonoProxy;
 
 #ifdef APDK_LJM1005
 class LJM1005Proxy;
+class LJP1XXXProxy;
 #endif
 
 #if defined(APDK_PSP100) && defined (APDK_DJ9xxVIP)
@@ -473,6 +474,11 @@ class Apollo21xxProxy;
 #ifdef APDK_APOLLO2XXX
 class Apollo2xxxProxy;
 #endif
+
+#ifdef APDK_QUICKCONNECT
+class QuickConnectProxy;
+#endif
+
 //DeviceRegistry
 //! Isolates device dependencies
 /*!
@@ -536,6 +542,7 @@ public:
 
 #ifdef APDK_LJM1005
     static LJM1005Proxy s_LJM1005Proxy;
+    static LJP1XXXProxy s_LJP1XXXProxy;
 #endif
 
 #if defined(APDK_PSP100) && defined (APDK_DJ9xxVIP)
@@ -624,6 +631,10 @@ public:
     static Apollo2xxxProxy s_Apollo2xxxProxy;
 #endif
 
+#ifdef APDK_QUICKCONNECT
+    static QuickConnectProxy s_QuickConnectProxy;
+#endif
+
 }; //DeviceRegistry
 
 
@@ -1149,6 +1160,24 @@ protected:
     DRIVER_ERROR SendCAPy (unsigned int iAbsY);
 }; // HeaderLJZjs
 
+//ClassName
+/*
+******************************************************************************/
+class HeaderQuickConnect : public Header
+{
+public:
+    HeaderQuickConnect (Printer *p, PrintContext *pc);
+    virtual DRIVER_ERROR Send ();
+    virtual DRIVER_ERROR FormFeed ();
+protected:
+    DRIVER_ERROR EndJob ();
+    DRIVER_ERROR SendCAPy (unsigned int iAbsY);
+}; // HeaderQuickConnect
+
+//ClassName
+/*
+******************************************************************************/
+
 class RasterSender : public Processor
 {
 friend class Header;
