@@ -1,7 +1,7 @@
 /*****************************************************************************\
   printer.cpp : Implimentation for the Printer class
 
-  Copyright (c) 1996 - 2001, Hewlett-Packard Co.
+  Copyright (c) 1996 - 2006, Hewlett-Packard Co.
   All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
@@ -74,6 +74,10 @@ Printer::Printer
 
     int i = 0; // counter
 
+#ifdef  APDK_LINUX
+    m_iNumPages = 0;
+#endif
+
     for (i = 0; i < MAX_PRINTMODES; i++)
     {
         pMode[i]=NULL;
@@ -586,7 +590,8 @@ DRIVER_ERROR Printer::Send
     residual = dwSendSize;
 
     if (bCheckForCancelButton &&
-        (ulBytesSentSinceCancelCheck >= CANCEL_BUTTON_CHECK_THRESHOLD) )
+        (ulBytesSentSinceCancelCheck >= CANCEL_BUTTON_CHECK_THRESHOLD) &&
+        (pSS->IOMode.bDevID))
     {
         ulBytesSentSinceCancelCheck = 0;
         char* tmpStr;
@@ -684,7 +689,8 @@ DRIVER_ERROR Printer::Send
             if (m_bStatusByPJL)
             {
                 status_reg = (BYTE) DISPLAY_PRINTING;
-                eDisplayStatus = ParseError (status_reg);
+                if (IOMode.bStatus)
+                    eDisplayStatus = ParseError (status_reg);
                 if (eDisplayStatus != DISPLAY_PRINTING)
                     write_error = IO_ERROR;
             }
