@@ -79,6 +79,11 @@
 #endif
 #ifdef APDK_LJM1005
 #include "ljm1005.h"
+#include "ljp1xxx.h"
+#endif
+
+#ifdef APDK_QUICKCONNECT
+#include "quickconnect.h"
 #endif
 
 APDK_BEGIN_NAMESPACE
@@ -109,6 +114,7 @@ LJZjsMonoProxy DeviceRegistry::s_LJZjsMonoProxy;
 
 #ifdef APDK_LJM1005
 LJM1005Proxy DeviceRegistry::s_LJM1005Proxy;
+LJP1XXXProxy DeviceRegistry::s_LJP1XXXProxy;
 #endif
 
 #if defined(APDK_PSP100) && defined (APDK_DJ9xxVIP)
@@ -197,6 +203,10 @@ Apollo21xxProxy DeviceRegistry::s_Apollo21xxProxy;
 Apollo2xxxProxy DeviceRegistry::s_Apollo2xxxProxy;
 #endif
 
+#ifdef APDK_QUICKCONNECT
+QuickConnectProxy DeviceRegistry::s_QuickConnectProxy;
+#endif
+
 DeviceRegistry::DeviceRegistry()
     : device(UNSUPPORTED)
 {
