@@ -24,6 +24,7 @@ import operator
 
 # Local
 from base.g import *
+from base.codes import *
 from base import utils, device
 from prnt import cups
 from ui_utils import load_pixmap
@@ -75,11 +76,11 @@ class FaxSendJobForm(QMainWindow):
         self.clearWState(Qt.WState_Polished)
         self.languageChange()
 
-        if self.device_uri and self.printer_name:
-            log.error("You may not specify both a printer (-p) and a device (-d).")
-            self.FailureUI(self.__tr("<p><b>You may not specify both a printer (-p) and a device (-d)."))
-            self.device_uri, self.printer_name = None, None
-            self.init_failed = True
+#        if self.device_uri and self.printer_name:
+#            log.error("You may not specify both a printer (-p) and a device (-d).")
+#            self.FailureUI(self.__tr("<p><b>You may not specify both a printer (-p) and a device (-d)."))
+#            self.device_uri, self.printer_name = None, None
+#            self.init_failed = True
 
         self.cups_printers = cups.getPrinters()
         log.debug(self.cups_printers)
@@ -100,11 +101,14 @@ class FaxSendJobForm(QMainWindow):
                 self.init_failed = True
 
         if not self.device_uri and not self.printer_name:
-            t = device.probeDevices(bus=bus, filter={'fax-type':(operator.gt, 0)})
+            t = device.probeDevices(bus=bus, filter={'fax-type':(operator.gt, FAX_TYPE_NONE)})
+            #print t
             probed_devices = []
 
             for d in t:
                 probed_devices.append(d.replace('hp:/', 'hpfax:/'))
+                
+            #print probed_devices
 
             probed_devices = utils.uniqueList(probed_devices)
             log.debug(probed_devices)
@@ -114,12 +118,19 @@ class FaxSendJobForm(QMainWindow):
             for d in probed_devices:
                 printers = []
                 for p in self.cups_printers:
+                    #print p.device_uri, d
                     if p.device_uri == d:
+                        #print "OK"
                         printers.append(p.name)
+                
                 devices[x] = (d, printers)
                 x += 1
                 max_deviceid_size = max(len(d), max_deviceid_size)
 
+            x = len(devices)
+            
+            #print devices
+            
             if x == 0:
                 from nodevicesform import NoDevicesForm
                 self.FailureUI(self.__tr("<p><b>No devices found.</b><p>Please make sure your device is properly installed and try again."))
@@ -132,33 +143,41 @@ class FaxSendJobForm(QMainWindow):
             else:
                 from chooseprinterdlg import ChoosePrinterDlg
                 dlg = ChoosePrinterDlg(self.cups_printers, ['hpfax'])
+                
                 if dlg.exec_loop() == QDialog.Accepted:
                     self.device_uri = dlg.device_uri
                 else:
                     self.init_failed = True
                     
-        self.dbus_avail, self.service = device.init_dbus()
+        self.dbus_avail, self.service, session_bus = device.init_dbus()
 
-        self.FaxView = ScrollFaxView(self.service, False, self.centralWidget(), self)
+        self.FaxView = ScrollFaxView(self.service, self.centralWidget(), self)
         self.FormLayout.addWidget(self.FaxView,0,0)
 
         if not self.init_failed:
-            try:
-                self.cur_device = device.Device(device_uri=self.device_uri, 
-                                                 printer_name=self.printer_name)
-            except Error, e:
-                log.error("Invalid device URI or printer name.")
-                self.FailureUI("<b>Invalid device URI or printer name.</b><p>Please check the parameters to hp-print and try again.")
+            if not self.device_uri or not self.device_uri.startswith("hpfax:"):
+                log.error("Invalid device URI: %s" % repr(device_uri))
+                self.FailureUI(self.__tr("<b>Invalid device URI %1.</b><p>Please check the parameters to hp-print and try again.").arg(repr(device_uri)));
                 self.init_failed = True
-
+            
             else:
-                self.device_uri = self.cur_device.device_uri
-                user_cfg.last_used.device_uri = self.device_uri
+                try:
+                    self.cur_device = device.Device(device_uri=self.device_uri, 
+                                                     printer_name=self.printer_name)
+                except Error, e:
+                    log.error("Invalid device URI or printer name.")
+                    self.FailureUI("<b>Invalid device URI or printer name.</b><p>Please check the parameters to hp-print and try again.")
+                    self.init_failed = True
 
-                log.debug(self.device_uri)
+                else:
+                    self.device_uri = self.cur_device.device_uri
+                    user_cfg.last_used.device_uri = self.device_uri
 
-                self.statusBar().message(self.device_uri)        
+                    log.debug(self.device_uri)
 
+                    self.statusBar().message(self.device_uri) 
+            
+            
         QTimer.singleShot(0, self.InitialUpdate)
 
 
@@ -179,6 +198,12 @@ class FaxSendJobForm(QMainWindow):
 
     def languageChange(self):
         self.setCaption(self.__tr("HP Device Manager - Send Fax"))
+        
+    def closeEvent(self, event):
+        #print "close"
+        #print self.FaxView.lock_file
+        utils.unlock(self.FaxView.lock_file)
+        event.accept()
 
     def SuccessUI(self):
         QMessageBox.information(self,
